@@ -62,17 +62,17 @@ extern char *strstr();	/* needed on sysV68 R3V7.1. */
 #define IMAP4		0	/* IMAP4 rev 0, RFC1730 */
 #define IMAP4rev1	1	/* IMAP4 rev 1, RFC2060 */
 
-static int count, seen, recent, unseen, deletions, imap_version, preauth; 
-static int expunged, expunge_period;
-static flag do_idle, idling;
+static int count, unseen, deletions, imap_version, preauth; 
+static int expunged, expunge_period, saved_timeout;
+static flag do_idle;
 static char capabilities[MSGBUFSIZE+1];
+static unsigned int *unseen_messages;
 
 int imap_ok(int sock, char *argbuf)
 /* parse command response */
 {
-    char buf [MSGBUFSIZE+1];
+    char buf[MSGBUFSIZE+1];
 
-    seen = 0;
     do {
 	int	ok;
 	char	*cp;
@@ -88,7 +88,7 @@ int imap_ok(int sock, char *argbuf)
 	/* interpret untagged status responses */
 	if (strstr(buf, "* CAPABILITY"))
 	    strncpy(capabilities, buf + 12, sizeof(capabilities));
-	if (strstr(buf, "EXISTS"))
+	else if (strstr(buf, "EXISTS"))
 	{
 	    count = atoi(buf+2);
 	    /*
@@ -98,31 +98,18 @@ int imap_ok(int sock, char *argbuf)
 	     * see an EXISTS.  Only after that will a tagged response be
 	     * shipped.  The idling flag also gets cleared on a timeout.
 	     */
-	    if (idling)
+	    if (stage == STAGE_IDLE)
 	    {
-		gen_send(sock, "DONE");
-		idling = FALSE;
+		/* we do our own write and report here to disable tagging */
+		SockWrite(sock, "DONE\r\n", 6);
+		if (outlevel >= O_MONITOR)
+		    report(stdout, "IMAP> DONE\n");
+
+		mytimeout = saved_timeout;
+		stage = STAGE_FETCH;
 	    }
 	}
-	if (strstr(buf, "RECENT"))
-	    recent = atoi(buf+2);
-	if (strstr(buf, "UNSEEN"))
-	{
-	    char	*cp;
-
-	    /*
-	     * Handle both "* 42 UNSEEN" (if tha ever happens) and 
-	     * "* OK [UNSEEN 42] 42". Note that what this gets us is
-	     * a minimum index, not a count.
-	     */
-	    unseen = 0;
-	    for (cp = buf; *cp && !isdigit(*cp); cp++)
-		continue;
-	    unseen = atoi(cp);
-	}
-	if (strstr(buf, "FLAGS"))
-	    seen = (strstr(buf, "SEEN") != (char *)NULL);
-	if (strstr(buf, "PREAUTH"))
+	else if (strstr(buf, "PREAUTH"))
 	    preauth = TRUE;
     } while
 	(tag[0] != '\0' && strncmp(buf, tag, strlen(tag)));
@@ -143,14 +130,7 @@ int imap_ok(int sock, char *argbuf)
 	while (isspace(*cp))
 	    cp++;
 
-	if (strncmp(cp, "PREAUTH", 2) == 0)
-	{
-	    if (argbuf)
-		strcpy(argbuf, cp);
-	    preauth = TRUE;
-	    return(PS_SUCCESS);
-	}
-	else if (strncmp(cp, "OK", 2) == 0)
+        if (strncmp(cp, "OK", 2) == 0)
 	{
 	    if (argbuf)
 		strcpy(argbuf, cp);
@@ -343,12 +323,18 @@ static int do_rfc1731(int sock, char *truename)
 	return PS_AUTHFAIL;
     }
 
+#ifdef __UNUSED__
+    /*
+     * Andrew H. Chatham <andrew.chatham@duke.edu> alleges that this check
+     * is not necessary and has consistently been messing him up.
+     */
     if (strcmp(tktuser, user) != 0) {
 	report(stderr, 
 	       _("principal %s in ticket does not match -u %s\n"), tktuser,
 		user);
 	return PS_AUTHFAIL;
     }
+#endif /* __UNUSED__ */
 
     if (tktinst[0]) {
 	report(stderr, 
@@ -873,7 +859,6 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
 
     /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
     capabilities[0] = '\0';
-    preauth = FALSE;
     if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
     {
 	/* UW-IMAP server 10.173 notifies in all caps */
@@ -911,17 +896,19 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
 	expunge_period = 1;
 
     /* 
-     * If either (a) we saw a PREAUTH token in the capability response, or
+     * If either (a) we saw a PREAUTH token in the greeting, or
      * (b) the user specified ssh preauthentication, then we're done.
      */
     if (preauth || ctl->server.preauthenticate == A_SSH)
-	return(PS_SUCCESS);
+    {
+        preauth = FALSE;  /* reset for the next session */
+        return(PS_SUCCESS);
+    }
 
     /* 
      * Handle idling.  We depend on coming through here on startup
      * and after each timeout (including timeouts during idles).
      */
-    idling = FALSE;
     if (strstr(capabilities, "IDLE") && ctl->idle)
     {
 	do_idle = TRUE;
@@ -1064,9 +1051,10 @@ static int imap_getrange(int sock,
 /* get range of messages to be fetched */
 {
     int ok;
+    char buf[MSGBUFSIZE+1], *cp;
 
     /* find out how many messages are waiting */
-    *bytes = recent = unseen = -1;
+    *bytes = -1;
 
     if (pass > 1)
     {
@@ -1080,7 +1068,12 @@ static int imap_getrange(int sock,
 	if (deletions && expunge_period != 1)
 	    ok = internal_expunge(sock);
 	count = -1;
-	idling = do_idle;
+	if (do_idle)
+	{
+	    stage = STAGE_IDLE;
+	    saved_timeout = mytimeout;
+	    mytimeout = 0;
+	}
 	if (ok || gen_transact(sock, do_idle ? "IDLE" : "NOOP"))
 	{
 	    report(stderr, _("re-poll failed\n"));
@@ -1088,9 +1081,10 @@ static int imap_getrange(int sock,
 	}
 	else if (count == -1)	/* no EXISTS response to NOOP/IDLE */
 	{
-	    count = recent = 0;
-	    unseen = -1;
+	    count = 0;
 	}
+	if (outlevel >= O_DEBUG)
+	    report(stdout, "%d messages waiting after re-poll\n", count);
     }
     else
     {
@@ -1102,24 +1096,63 @@ static int imap_getrange(int sock,
 	    report(stderr, _("mailbox selection failed\n"));
 	    return(ok);
 	}
+	else if (outlevel >= O_DEBUG)
+	    report(stdout, "%d messages waiting after first poll\n", count);
     }
 
     *countp = count;
 
-    /*
-     * Note: because IMAP has an is_old method, this number is used
-     * only for the "X messages (Y unseen)" notification.  Accordingly
-     * it doesn't matter much that it can be wrong (e.g. if we see an
-     * UNSEEN response but not all messages above the first UNSEEN one
-     * are likewise).
-     */
-    if (unseen >= 0)		/* optional, but better if we see it */
-	*newp = count - unseen + 1;
-    else if (recent >= 0)	/* mandatory */
-	*newp = recent;
-    else
-	*newp = -1;		/* should never happen, RECENT is mandatory */ 
+    /* OK, now get a count of unseen messages and their indices */
+    if (!ctl->fetchall && count > 0)
+    {
+	if (unseen_messages)
+	    free(unseen_messages);
+	unseen_messages = xmalloc(count * sizeof(unsigned int));
+	memset(unseen_messages, 0, count * sizeof(unsigned int));
+	unseen = 0;
 
+	gen_send(sock, "SEARCH UNSEEN");
+	do {
+	    ok = gen_recv(sock, buf, sizeof(buf));
+	    if (ok != 0)
+	    {
+		report(stderr, _("search for unseen messages failed\n"));
+		return(PS_PROTOCOL);
+	    }
+	    else if ((cp = strstr(buf, "* SEARCH")))
+	    {
+		char	*ep;
+
+		cp += 8;	/* skip "* SEARCH" */
+
+		while (*cp && unseen < count)
+		{
+		    /* skip whitespace */
+		    while (*cp && isspace(*cp))
+			cp++;
+		    if (*cp) 
+		    {
+			/*
+			 * Message numbers are between 1 and 2^32 inclusive,
+			 * so unsigned int is large enough.
+			 */
+			unseen_messages[unseen]=(unsigned int)strtol(cp,&ep,10);
+
+			if (outlevel >= O_DEBUG)
+			    report(stdout, 
+				   _("%u is unseen\n"), 
+				   unseen_messages[unseen]);
+		
+			unseen++;
+			cp = ep;
+		    }
+		}
+	    }
+	} while
+	    (tag[0] != '\0' && strncmp(buf, tag, strlen(tag)));
+    }
+
+    *newp = unseen;
     expunged = 0;
 
     return(PS_SUCCESS);
@@ -1186,13 +1219,22 @@ static int imap_getsizes(int sock, int count, int *sizes)
 static int imap_is_old(int sock, struct query *ctl, int number)
 /* is the given message old? */
 {
-    int ok;
+    flag seen = TRUE;
+    int i;
 
-    /* expunges change the fetch numbers */
-    number -= expunged;
+    /* 
+     * Expunges change the fetch numbers, but unseen_messages contains
+     * indices from before any expungees were done.  So neither the
+     * argument nor the values in message_sequence need to be decremented.
+     */
 
-    if ((ok = gen_transact(sock, "FETCH %d FLAGS", number)) != 0)
-	return(PS_ERROR);
+    seen = TRUE;
+    for (i = 0; i < unseen; i++)
+	if (unseen_messages[i] == number)
+	{
+	    seen = FALSE;
+	    break;
+	}
 
     return(seen);
 }
@@ -1246,6 +1288,20 @@ static int imap_fetch_body(int sock, struct query *ctl, int number, int *lenp)
      * In that case, marking the seen flag is the only way to prevent the
      * message from being re-fetched on subsequent runs (and according
      * to RFC2060 p.43 this fetch should set Seen as a side effect).
+     *
+     * According to RFC2060, and Mark Crispin the IMAP maintainer,
+     * FETCH %d BODY[TEXT] and RFC822.TEXT are "functionally 
+     * equivalent".  However, we know of at least one server that
+     * treats them differently in the presence of MIME attachments;
+     * the latter form downloads the attachment, the former does not.
+     * The server is InterChange, and the fool who implemented this
+     * misfeature ought to be strung up by his thumbs.  
+     *
+     * When I tried working around this by disable use of the 4rev1 form,
+     * I found that doing this breaks operation with M$ Exchange.
+     * Annoyingly enough, Exchange's refusal to cope is technically legal
+     * under RFC2062.  Trust Microsoft, the Great Enemy of interoperability
+     * standards, to find a way to make standards compliance irritating....
      */
     switch (imap_version)
     {
@@ -1381,6 +1437,12 @@ static int imap_logout(int sock, struct query *ctl)
     if (deletions)
 	internal_expunge(sock);
 
+#ifdef USE_SEARCH
+    /* Memory clean-up */
+    if (unseen_messages)
+	free(unseen_messages);
+#endif /* USE_SEARCH */
+
     return(gen_transact(sock, "LOGOUT"));
 }
 
